program BouncingBallsSDL;
uses SDL, SDL_image, matrix, SysUtils, math;

function MinAbsIndex (V : Tvector2_Double) : Integer;
begin
   if abs (V.data[0]) <= abs (V.data[1]) then result := 0
   else result := 1
end;

const
   G = 0.1;
   DISSIPATE : Tvector2_Double_data = (0.0, 0.0001);
   BackgroundFile = 'background.bmp';
   Ball0File = 'ball.bmp';
   Ball1File = 'ball2.bmp';
var
   screen, background : PSDL_Surface;
   ball : array[0..1] of PSDL_Surface;
   
   procedure DrawBallOrShadow (ball : PSDL_Surface; shadow : Boolean;
                               const PixX, PixY : Integer);
   var
      SrcRect, DstRect : SDL_Rect;
   begin
      if (PixX >= 0) and (PixY >= 0)
         and (PixX + ball^.w < screen^.w) and (PixY + ball^.h < screen^.h)
      then begin
         with SrcRect do begin
            x := 0; y := 0; w := ball^.w; h := ball^.h
         end;
         with DstRect do begin
            x := PixX; y := PixY; w := PixX + ball^.w; h := PixY + ball^.h
         end;
         if shadow then begin
            if SDL_BlitSurface (background, @DstRect, screen, @DstRect) < 0
            then raise Exception.Create('BlitSurface shadow error')
         end else begin
            if SDL_BlitSurface (ball, @SrcRect, screen, @DstRect) < 0
            then raise Exception.Create('BlitSurface ball error')
         end
      end
   end;
   
   function Quitted : Boolean;
   var event : TSDL_Event;
   begin
      Quitted := False;
      while SDL_PollEvent (@event) = 1
      do
         case event.type_ of
            SDL_KeyDown:
               Quitted := Quitted or (event.key.keysym.sym = LongWord ('q'));
            SDL_QuitEv: Quitted := True;
         end;
   end;
   
var
   ScreenSize : array[0..1] of Double;
   BallRad : array[0..1] of Double;
   BallSumRad : Double;
   bn : Integer;                {Ball number.}
   
   procedure Main;
   var
      Dissipation : Tvector2_double;
      R, V : array[0..1] of Tvector2_double;
      NR, DR : Tvector2_double;
      i : Integer;
   begin
      Dissipation.data := DISSIPATE;
      FillByte (R, SizeOf(R), 0);
      FillByte (V, SizeOf(V), 0);
      R[0].Init (0.48 * screen^.w / 2, 0.86 * screen^.h / 2);
      V[0].Init (0.007 * screen^.w / 2, 0.011 * screen^.h / 2);
      R[1].Init (0.48 * screen^.w / 2, 0.46 * screen^.h / 2);
      V[1].Init (-0.007 * screen^.w / 2, 0.011 * screen^.h / 2);
      repeat
         for bn := 0 to 1 do
         begin
            NR := R[bn] - BallRad[bn];
            DrawBallOrShadow (ball[bn], True,
                              Round (NR.data[0]), Round (NR.data[1]));
         end;         
         for bn := 0 to 1 do
         begin
            V[bn] *=  -Dissipation + 1;
            V[bn].data[1] += G;
            
            NR := R[bn] + V[bn];
            DR := NR - R[1-bn];
            for i := 0 to 1 do
               if (NR.data[i] - BallRad[bn] < 0)
               then begin
                  R[bn].data[i] := BallRad[bn];
                  V[bn].data[i] := abs (V[bn].data[i])
               end else if (NR.data[i] + BallRad[bn] >= ScreenSize[i])
               then begin
                  R[bn].data[i] := ScreenSize[i] - BallRad[bn];
                  V[bn].data[i] := -abs (V[bn].data[i])
               end;
            
            if DR.Length < BallSumRad
            then begin
               i := MinAbsIndex (DR);
               V[bn].data[i] := sign (DR.data[i]) * abs (V[bn].data[i])
            end;
            
            R[bn] := R[bn] + V[bn];
         
            NR := R[bn] - BallRad[bn];
            DrawBallOrShadow (ball[bn], False,
                              Round (NR.data[0]), Round (NR.data[1]));
         end;
         {WriteLn ('x1=',R[0].data[0]:0:1, '; y1=',R[0].data[1]:0:1,
                  '; x2=',R[1].data[0]:0:1, '; y2=',R[1].data[1]:0:1,
                  '; vx1=',V[0].data[0]:0:1, '; vy1=',V[0].data[1]:0:1,
                  '; vx2=',V[1].data[0]:0:1, '; vy2=',V[1].data[1]:0:1);}
         SDL_Flip (screen);
         SDL_Delay (10)
      until Quitted;
   end;
   
begin
   WriteLn ('Initializing SDL.');
   try try
      if SDL_Init(SDL_INIT_VIDEO) < 0
      then raise Exception.Create('Couldn''t initialize SDL');
      background := IMG_Load (BackgroundFile);
      if background = nil
      then raise Exception.Create('Couldn''t load '+ BackgroundFile);
      ScreenSize[0] := background^.w;
      ScreenSize[1] := background^.h;
      screen := SDL_SetVideoMode(background^.w, background^.h,
                                 background^.format^.BitsPerPixel,
                                 SDL_HWSURFACE or SDL_DOUBLEBUF);
      if screen = nil
      then raise Exception.Create('Couldn''t set video mode');
      ball[0] := IMG_Load (Ball0File);
      if ball[0] = nil
      then raise Exception.Create('Couldn''t load '+ Ball0File);
      ball[1] := IMG_Load (Ball1File);
      if ball[1] = nil
      then raise Exception.Create('Couldn''t load '+ Ball1File);
      for bn := 0 to 1 do
         BallRad[bn] := (ball[bn]^.w + ball[bn]^.h) / 4;
      BallSumRad := BallRad[0] + BallRad[1];
      
      if SDL_BlitSurface(background, nil, screen, nil) < 0
      then raise Exception.Create('BlitSurface background error');
      Main;
      
   except
      on E: Exception do
         WriteLn ('ERROR: ', E.Message, ' -- ', SDL_GetError);
   end
   finally
      if background <> nil then SDL_FreeSurface(background);
      for bn := 0 to 1 do
         if ball[bn] <> nil then SDL_FreeSurface(ball[bn]);
      if screen <> nil then SDL_FreeSurface(screen);
      SDL_Quit
   end
end.